#include "adammethod.h"

AdamMethod::AdamMethod(Problem *p)
{
    myProblem = p;
    alpha =0.0001;
    beta1=0.9;
    beta2=0.99;
    epsilon =0.01;
    int n = myProblem->getDimension();
    g.resize(n);
    m.resize(n);
    u.resize(n);
    mtilde.resize(n);
    utilde.resize(n);
}

void    AdamMethod::setAlpha(double a)
{
    if(a>0) alpha = a;
}

double  AdamMethod::getAlpha() const
{
    return alpha;
}

void    AdamMethod::setBeta1(double b)
{
    if(b>0)
    beta1 = b;
}

double  AdamMethod::getBeta1() const
{
    return beta1;
}

void    AdamMethod::setBeta2(double b)
{
    if(b>0)
    beta2 = b;
}

double  AdamMethod::getBeta2() const
{
    return beta2;
}

void    AdamMethod::setEpsilon(double e)
{
    if(e>0)
        epsilon = e;
}

double  AdamMethod::getEpsilon() const
{
    return epsilon;
}

void    AdamMethod::Solve()
{
    int n = myProblem->getDimension();
    int k=0;
    xpoint = myProblem->getSample();
    for(int i=0;i<n;i++)
    {
        u[i]=0.0;
        m[i]=0.0;
    }
    while(true)
    {
        g = myProblem->gradient(xpoint);
        for(int i=0;i<n;i++)
        {
            m[i]=beta1* m[i]+(1-beta1)*g[i];
            u[i]=beta2*u[i]+(1-beta2)*g[i]*g[i];
            mtilde[i]=1.0/(1-pow(beta1,k+1))*m[i];
            utilde[i]=1.0/(1-pow(beta2,k+1))*u[i];
            xpoint[i]=xpoint[i]-alpha * mtilde[i]/(sqrt(utilde[i])+epsilon);
        }
        k=k+1;
        ypoint = myProblem->funmin(xpoint);
        printf("ADAM. ITER=%4d VALUE=%20.10lg\n",k,ypoint);
        double f = myProblem->grms(xpoint);
        if(f<1e-3) break;
    }
}

Data    AdamMethod::getBest() const
{
    return xpoint;
}

AdamMethod::~AdamMethod()
{
    //nothing here
}
